// Time-stamp: <05-Sep-2002 13:22:26 jwerner>
/*====================================================================
//
// gpl2ornt - convert gpl to "orientiering" file
//
//     by: John E. Werner
//         03-Feb-1998
//
//====================================================================
//
// Purpose:
//
// System Subsystem:
//         Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1998, 2001, 2002 John E. Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   03-Feb-1998  JEWerner	Created.
//
//------------------------------------------------------------------*/
#pragma pack(1)

#include <iostream.h>
#include <strstream.h>
#include <fstream.h>
#include <string>
#include <stdlib.h>
#include "gpldata.h"
#include "GFC.h"

typedef unsigned short UShort;
typedef unsigned long  ULong;

#define FT_PER_METER	3.281
#define SEC_PER_HOUR	3600.0
#define MI_PER_METER	0.0006214

std::string& WriteData(GPL_Data_Point &data, double distance, bool dmsP);

//--------------------------------------------------------------------
// main(...)
//
//   Description:
//	The main execution loop
//
//   Params:
//	int	argc	- the number of arguments
//	char	**argv	- the argnuments
//
//   Returns:
//	int	0	No Error
//		!0	Error
//			-1 	Failed to open input file
//			-2	Failed to open output file
//			-3 	Invalid arguments
//--------------------------------------------------------------------

int main(int argc, char **argv)
{
   if (argc < 2)				// check number of args
   {
      cout << "Usage: gpl2ornt [-m] <file1.gpl> ... <fileN.gpl>" << endl
           << "\t -m = Convert coordinates to degrees:min:sec"
           << endl;
      return -3;
   }

   int 	returnResult = 0;		// our return value
   std::string 	dmsString("-m");		// the option to cause dms
   bool 	dmsFormat = false;		// use degree:m:sec format?

   for (int i = 1; i < argc; i++)		// process command line
   {
      std::string      inName(argv[i]);

      if (inName == dmsString)
      {
         dmsFormat = true;
         continue;
      }

      cout << "# Opening " << inName << endl;
      ifstream in(inName.c_str(), ios::binary);
      if (!in)
      {
         cout << "\"" << inName << "\" - Open Failed!!!" << endl;
         returnResult = -1;
         continue;				// try next file
      }

      cout << "# Latitude, Longitude, Heading, Distance, Speed"
           << endl;

      bool done = false;
      int  num = 1;

      GPL_Data_Point 		data;
      GPL_Data_Point 		oldData;
      CPolarCoordinate	dataCoord;
      CPolarCoordinate	oldDataCoord;
      CEarth		earth;

      // read first data point
      in.read((char *) &oldData, sizeof(oldData));
      if (in.fail())
      {
         done = true;
      }

      while (!done)
      {
//         bool 	writeIt = false;
         double	dist = 0.0;

         if (in.eof())				// see if input file done
         {
            done = true;
            cout << WriteData(oldData, dist, dmsFormat) << endl;
            break;
         }

         // get data point
         in.read((char *) &data, sizeof(data));
         if (in.fail())
         {
            done = true;
            cout << WriteData(oldData, dist, dmsFormat) << endl;
            break;
         }

         oldDataCoord.Set(oldData.GetLattitude(),
                          oldData.GetLongitude(),
                          oldData.GetAltitude() / FT_PER_METER);

         dataCoord.Set(data.GetLattitude(),
                       data.GetLongitude(),
                       data.GetAltitude() / FT_PER_METER);

         double headingTo;
         double headingFrom;

         dist = earth.GetSurfaceDistance(oldDataCoord, dataCoord,
                                         &headingTo, &headingFrom);

         dist = dist * MI_PER_METER;		// convert to miles
         ULong ultime = data.GetTime() - oldData.GetTime();
         double time = (double) ultime / SEC_PER_HOUR;

         double speed = dist / time;		// figure out mph

         oldData.SetHeading(headingTo);	// save value
         oldData.SetSpeed(speed);		// save value

         // send out the data
         cout << WriteData(oldData, dist, dmsFormat) << endl;

         oldData = data;

         num++;

      } // while(!done)

      // close stuff and leave
      in.close();
      cout << "# ... done.  Processed " << num << " data points." << endl;
   } // for(i = 1; i < argc; i++)

   return returnResult;
} // main(...)

//--------------------------------------------------------------------
// WriteData(...)
//
//   Description:
//	write the data
//
//   Params:
//	GPL_Data_Point	data		- the data point
//	double		distance	- the distance to next
//	bool		dmsP		- dms format?
//
//   Returns:
//	ostream &
//
//   Notes:
//	"Latitude, Longitude, Heading, Distance, Speed"
//--------------------------------------------------------------------
std::string& WriteData(GPL_Data_Point &data, double distance, bool dmsP)
{
    strstream os;

    // set formatting (printf sure looks nice right now)
    os.setf( ios::fixed );
    os.setf( ios::right );    

    if (dmsP)					// if dms format
    {
	double d, m, s;

	// do latitude
	CMath::ConvertDecimalDegreesToDegreesMinutesSeconds(data.GetLattitude(),
							    d, m, s);
	os.precision(0);	
	os.unsetf( ios::showpoint );
	os.fill(' ');
	os.width(4);	
	os << d << ":";
	os.fill('0');
	os.width(2);	
	os << m << ":";
	os.setf( ios::showpoint );
	os.precision(4);
	os.width(7);	
	os << s << ", ";

	// do longitude
	CMath::ConvertDecimalDegreesToDegreesMinutesSeconds(data.GetLongitude(),
							    d, m, s);
	os.precision(0);	
	os.unsetf( ios::showpoint );
	os.fill(' ');
	os.width(4);	
	os << d << ":";
	os.fill('0');
	os.width(2);	
	os << m << ":";
	os.setf( ios::showpoint );
	os.precision(4);
	os.width(7);	
	os << s << ", ";
    }
    else					// not dms format
    {
	os.precision(8);
	os.setf( ios::showpoint );
	os << data.GetLattitude() << ", " << data.GetLongitude() << ", ";
    }

    // altitude, heading
    os.fill(' ');    
    os.setf( ios::showpoint );
    os.width(8);
    os.precision(4);
    os << data.GetHeading() << ", ";

    // distance, speed
    os.width(9);
    os.precision(4);
    os << distance << ", ";
    os.width(9);
    os << data.GetSpeed() << '\0';

    static std::string str;
    str = os.str();
    return str;
} // WriteData(...)
