// Time-stamp: <05-Sep-2002 13:22:27 jwerner>
/*====================================================================
//
// respeed.cc - Recalculate speeds based on positions and time
//
//     by: John E. Werner
//         03-Feb-1998
//
//====================================================================
//
// Purpose:
//
// System Subsystem:
//         Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1998, 2001, 2002 John E. Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   03-Feb-1998  JEWerner	Created.
//   16-Aug-2002  JEWerner If a speed is give, we retime based on the
//                         speed given.
//
//------------------------------------------------------------------*/
#pragma pack(1)

#include <iostream.h>
#include <fstream.h>
#include <string>
#include <stdlib.h>
#include "gpldata.h"
#include "GFC.h"

typedef unsigned short UShort;
typedef unsigned long  ULong;

#define FT_PER_METER	3.281
#define SEC_PER_HOUR	3600.0
#define MI_PER_METER	0.0006214

//--------------------------------------------------------------------
// main(...)
//
//   Description:
//	The main execution loop
//
//   Params:
//	int	argc	- the number of arguments
//	char	**argv	- the argnuments
//
//   Returns:
//	int	0	No Error
//		!0	Error
//			-1 	Failed to open input file
//			-2	Failed to open output file
//			-3 	Invalid arguments
//--------------------------------------------------------------------

int main(int argc, char **argv)
{
   double givenSpeed(0.0);                // init givenSpeed to 0.0

   if (argc != 3)                         // check number of args
   {
      if (argc == 4)                      // new_mph_speed provided?
      {
         givenSpeed = strtod(argv[3], 0); // get the speed
         cout << "Setting Speed to " << givenSpeed << " mph." << endl;
      }
      else
      {
         cout << "Usage: respeed <input.gpl> <output.gpl> [<new_mph_speed>]"
              << endl;
         return -3;
      } // else .. if (argc == 4)
   } // if (argc != 3)

   std::string      outName(argv[2]), inName(argv[1]);

   cout << "Opening " << inName << endl;
   ifstream in(inName.c_str(), ios::binary);
   if (!in)
   {
      cout << "\"" << inName << "\" - Open Failed!!!" << endl;
      return -1;
   }

   cout << "Opening " << outName << endl;
   ofstream out(outName.c_str(), ios::binary | ios::trunc);
   if (!out)
   {
      cout << "\"" << outName << "\" - Open Failed!!!" << endl;
      in.close();
      return(-2);
   }

   cout << "Working...";
   bool done = false;
   int  num = 1;

   GPL_Data_Point data;
   GPL_Data_Point oldData;
   CPolarCoordinate	dataCoord;
   CPolarCoordinate	oldDataCoord;
   CEarth		earth;

   // read first data point
   in.read((char *) &oldData, sizeof(oldData));
   if (in.fail())
   {
      done = true;
   }


   while (!done)
   {
//	bool writeIt = false;

      if (in.eof())				// see if input file done
      {
         done = true;
         out.write((char *) &oldData, sizeof(oldData)); // write it
         break;
      }

      // get data point
      in.read((char *) &data, sizeof(data));
      if (in.fail())
      {
         done = true;
         out.write((char *) &oldData, sizeof(oldData)); // write it
         break;
      }

      oldDataCoord.Set(oldData.GetLattitude(),
                       oldData.GetLongitude(),
                       oldData.GetAltitude() / FT_PER_METER);

      dataCoord.Set(data.GetLattitude(),
                    data.GetLongitude(),
                    data.GetAltitude() / FT_PER_METER);

      double headingTo;
      double headingFrom;

      double dist = earth.GetSurfaceDistance(oldDataCoord, dataCoord,
                                             &headingTo, &headingFrom);

      dist = dist * MI_PER_METER;		// convert to miles

      if (givenSpeed != 0.0)
      {
         // We have been given a speed, so calculate the time between
         double time = (dist / givenSpeed) * SEC_PER_HOUR;

         oldData.SetSpeed(givenSpeed);
         data.SetTime(oldData.GetTime() + time);
      }
      else
      {
         // Since no speed was give, we need to figure out a new speed between
         // the points.
         ULong ultime = data.GetTime() - oldData.GetTime();
         double time = (double) ultime / SEC_PER_HOUR;

         double speed = dist / time;		// figure out mph

         oldData.SetHeading(headingTo);   // save value
         oldData.SetSpeed(speed);         // save value
      } // else .. if (givenSpeed == 0.0)


      out.write((char *) &oldData, sizeof(oldData)); // write it

      oldData = data;

      num++;
      if (num % 100 == 0)
         cout << ".";

   } // while(!done)

   // close stuff and leave
   in.close();
   out.close();

   cout << "done." << endl;
   cout << "Wrote " << num << " data points." << endl;

   return 0;
} // main(...)
