/************************************************************************

Project:      Cetus GPS track export
Description:  This file contains routines for exporting data from a track
              file generated by Cetus GPS to a text file. A description
              of the track file is in tracklog.h

              Please notice that this file has been modified for
              compilation using the DOS Turbo C 2.01 compiler. The read
              function has been replaced by _read, the open function
              replaced by _open and the include of unistd.h has been
              disabled.

File:         tracklog.c
Author:       Kjeld Jensen <kjeld@cetus.dk>
Homepage:     http://www.cetus.dk/gps

Revision history:

Date     Vers  Author Description
-------- ----- ------ ---------------------------------------------------
01-06-01  1.0    KJ   Created, supports Cetus GPS 1.0b1
05-06-01  1.01   KJ   read replaced by _read and open replaced by _open
                      Changed handling of decimals for speed and HDOP
04-07-01  1.10   KJ   Changed to support Cetus GPS 1.0b9, added optional
                      formatting of position, included constant.h
                      which contains unit conversion constants used in
                      Cetus GPS.
29-09-01  1.20   KJ   Added timeUTC supported by Cetus GPS 1.0b13
                      added conditioned compilation in order to support
                      the different compilers

*************************************************************************
COPYRIGHT

Cetus GPS is copyrighted (c) 2001 by

Kjeld Jensen
Niels Bohrs Alle 23, apt. 2006
DK-5230 Odense M
Denmark
Email: gps@cetus.dk
http://www.cetus.dk/gps


COPYRIGHT INFORMATION
This source code is a part of the Cetus GPS project. The copyright stated
above includes all parts of the Cetus GPS project including but not
limited to binaries, source code, databases, documentation, homepage
information and design.

The following copyright information applies to the source code routines
published for the purpose of Cetus GPS data import and export and
binaries that includes a part of this source code. It does not apply to
the Cetus GPS project in general or other parts of the Cetus GPS project.

Redistribution of this source code for commercial or non-commercial use
in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

1. Copyright of the source code remain Kjeld Jensen's and as such any
   copyright notices in the source code are not to be removed.

2. Redistributions of the source code with or without modifications must
   retain the copyright notice, this source code copyright information
   and the disclaimer.

3. The licence and distribution terms for any available version or
   derivative of this source code cannot be changed. i.e. this source
   code cannot be copied and put under any other distribution licence.

4. The product documentation must include an acknowledgement where
   appropriate. A paragraph similar to the following is acceptable:
   "This product includes software written by Kjeld Jensen
   <kjeld@cetus.dk>"

Revised July 1, 2001


DISCLAIMER
All parts of the Cetus GPS project are provided by the copyright holder
as is, and any express or implied warranties, including, but not limited
to, the implied warranties of merchantability and fitness for a
particular purpose are disclaimed.

In no event skal the author or contributors be liable for any direct,
indirect, incidental, special, exemplary or consequential damages
(including but not limited to procurement of substitute goods or
services, loss of use, data, or profits, or business interruption)
however caused and on any theory of liability, whether in contract,
strict liability, or tort (including negligence or otherwise) arising
in any way out of the use of any parts of the Cetus GPS project, even
if advised of the possibility of such damage.

Revised July 1, 2001

************************************************************************/

/* compiler and platform related defines */
#include "global.h"

/* System includes */
#include <fcntl.h>     /* we need access to the open function */
#include <iostream>

/* Application includes */
#include "tracklog.h" /* defines shared with Cetus GPS */
#include "track.h"    /* defines for this app only */

/***********************************************************************/
/* Common variables */

static long trackCounter;

/***********************************************************************/
int track_init (ifstream &in, struct init_type *init)
{
	int err = 0;
	int i,j;
	unsigned char s[TrackRecordSize] = "x";
	int found = 0;

   /* try to open the track file */
   /* get the first record */
   /* search track file until EOF or first id character found */
   int   state = 0;

   do
   {
      while (!in.eof())
      {
         const char DBLK[5] = "DBLK";

         in.read(s,1);

         if (s[0] == DBLK[state])         // match characater?
         {                                // ... yes
            if (++state >= 4)             // ... inc state, if state >= 4
               break;                     // ... ... leave
         }
         else
            state = 0;                    // failed match, so start over
      } // while (!eof.());

      cout << "Found DBLK" << endl;

      in.read(s, 4);                      // read 4 "not used" bytes

      in.read(s, TrackRecordSize);        // get record

      if (!in.eof()
          && s[trkInfoID1] == 'C'
          && s[trkInfoID2] == 'G'
          && s[trkInfoVersion] == TrackVersion
          )
      {

         /* retrieve data */
         init->interval = s[trkInfoInterval];
         init->gps = (char) s[trkInfoGPSHi] << 8 | s[trkInfoGPSLo];
         init->year = s[trkInfoYear];
         init->month = s[trkInfoMonth];
         init->day = s[trkInfoDay];
         init->hour = s[trkInfoHour];
         init->minute = s[trkInfoMinute];
         init->second = s[trkInfoSecond];
         init->decSecond = s[trkInfoDecSecond];
         init->timeZone = s[trkInfoTimeOffset];

         // read in a block.  If we got an eof, then we did not find what we
         // expected.
         found = !(in.read(s, TrackRecordSize)).eof();

         for (i=0; i<TrackRecordSize; i++)
            init->desc[i] = s[i];

         // read in a block and see if we got what we expected.
         found = !(in.read(s,TrackRecordSize)).eof();

         for (i=TrackRecordSize, j=0; j<4; i++, j++)
            init->desc[i] = s[j];

         trackCounter = 3;
      } // if (... Found 'CG' record ...)
      else
         break;
   } while (!in.eof() && !found);
   if (!found)
      err = 2; /* not a track file */

   return err;
}
/***********************************************************************/
int track_record (ifstream &in, struct rec_type &rec)
{
	unsigned char s[TrackRecordSize];

   /* read the next record from the track file */
   in.read(s, TrackRecordSize);

	/* if we got a complete record */
	if (!in.eof())
   {
      unsigned short max_hour = 0;
      unsigned short rolled_days = 0;

      trackCounter++;

		rec.fix = s[trkStatus] >> 5;
		rec.hour = s[trkHour] & 0x1F + rolled_days * 24;
		rec.minute = s[trkMinute];
		rec.second = s[trkSecond];
      rec.decSecond = s[trkDecSecond];

      if (rec.hour < max_hour)
      {
         rolled_days++;
         rec.hour += 24;
      }

      if (rec.hour > max_hour)
         max_hour = rec.hour;

		rec.lat =  ((long) ((s[trkLatHiHi] << 8) | s[trkLatHiLo]) << 16)
                 | ((unsigned int) ((s[trkLatLoHi] << 8) | s[trkLatLoLo]));
		rec.lon =  ((long) ((s[trkLonHiHi] << 8) | s[trkLonHiLo]) << 16)
                 | ((unsigned int) ((s[trkLonLoHi] << 8) | s[trkLonLoLo]));

		rec.altitude =  ((long) ((s[trkAltHiHi] << 8) | s[trkAltHiLo]) << 16)
                      | ((unsigned int) ((s[trkAltLoHi] << 8) | s[trkAltLoLo]));

		rec.speed = s[trkSpeedHi] << 8 | s[trkSpeedLo];
		rec.course = s[trkCourseHi] << 8 | s[trkCourseLo];

		rec.sat = s[trkSat];
		rec.hdop = s[trkHDOP];

      // see if we need to do a skip.
      if (!in.eof() && ((trackCounter) % TrackRecords4096) == 0)
         in.read(s, TrackExportSkip);

//      cout //<< "fix:       " << rec.fix       << endl
//           << "year:      " << rec.year      << endl
//           << "month:     " << rec.month     << endl
//           << "day:       " << rec.day       << endl
//           << "hour:      " << rec.hour      << endl
//           << "minute:    " << rec.minute    << endl
//           << "second:    " << rec.second    << endl
//         ;
//           << "decSecond: " << rec.decSecond << endl
//           << "sat:       " << rec.sat       << endl
//           << "hdop:      " << rec.hdop      << endl
//           << "lat:       " << rec.lat       << endl
//           << "lon:       " << rec.lon       << endl
//           << "altitude:  " << rec.altitude  << endl
//           << "speed:     " << rec.speed     << endl
//           << "course:    " << rec.course    << endl;
   }
   return in.eof();
}
