/************************************************************************

Project:      Cetus GPS track export
Description:  This header file contains defines that are shared between
              Cetus GPS and the Cetus Track export program. There is
              also a description of the Cetus GPS track file format.
File:         tracklog.h
Author:       Kjeld Jensen <kjeld@cetus.dk>
Homepage:     http://www.cetus.dk/gps

Revision history:

Date     Vers  Author Description
-------- ----- ------ ---------------------------------------------------
01-06-01 1.0b1   KJ   Created
01-07-01 1.0b9   KJ   Changed lat, lon and alt to 32 bit integer, added
                      a special information record to the beginning of
                      the track log.
26-07-01 1.0b13  KJ   Changed local time to UTC. Added user
                      preference UTC time offset to the info record.
12-02-02 1.1b8   KJ   Removed datum and north information, changed
					  interval information (version 3)

*************************************************************************
COPYRIGHT

Cetus GPS
Copyright (c) 2001-2002

Kjeld Jensen
Niels Bohrs Alle 23, apt. 2006
DK-5230 Odense M
Denmark
Email: gps@cetus.dk
http://www.cetus.dk/gps


COPYRIGHT INFORMATION
This source code is a part of the Cetus GPS project. The copyright stated
above includes all parts of the Cetus GPS project including but not
limited to binaries, source code, databases, documentation, homepage
information and design.

The following copyright information applies to the source code routines
published for the purpose of Cetus GPS data import and export and
binaries that includes a part of this source code. It does not apply to
the Cetus GPS project in general or other parts of the Cetus GPS project.

Redistribution of this source code for commercial or non-commercial use
in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

1. Copyright of the source code remain Kjeld Jensen's and as such any
   copyright notices in the source code are not to be removed.

2. Redistributions of the source code with or without modifications must
   retain the copyright notice, this source code copyright information
   and the disclaimer.

3. The licence and distribution terms for any available version or
   derivative of this source code cannot be changed. i.e. this source
   code cannot be copied and put under any other distribution licence.

4. The product documentation must include an acknowledgement where
   appropriate. A paragraph similar to the following is acceptable:
   "This product includes software written by Kjeld Jensen
   <kjeld@cetus.dk>

Revised July 1, 2001


DISCLAIMER
All parts of the Cetus GPS project are provided by the copyright holder
as is, and any express or implied warranties, including, but not limited
to, the implied warranties of merchantability and fitness for a
particular purpose are disclaimed.

In no event skal the author or contributors be liable for any direct,
indirect, incidental, special, exemplary or consequential damages
(including but not limited to procurement of substitute goods or
services, loss of use, data, or profits, or business interruption)
however caused and on any theory of liability, whether in contract,
strict liability, or tort (including negligence or otherwise) arising
in any way out of the use of any parts of the Cetus GPS project, even
if advised of the possibility of such damage.

Revised July 1, 2001

************************************************************************/

/************************************************************************
FILE DATA DESCRIPTION

The track file starts with a header and then follows a set of 4096 bytes
data blocks each initiated with the characters 'DBLK' and a 4 byte block
id.

Cetus GPS writes the track records after the block header. The 4096
bytes data block is filled with empty bytes at the end, so each data
block begins with a new record.

The first three records in the track file has a special format, which is
described below.

************************************************************************/

/* Cetus GPS track version number */
#define TrackVersion        3

/* size of each record in bytes */
#define TrackRecordSize     22

/* the number of track records in a 4096 block */
#define TrackRecords4096    186

/* the number of bytes to skip at block end when writing */
#define TrackRestSkip       4

/* the number of bytes to skip at block end when exporting */
#define TrackExportSkip     12

/************************************************************************
TRACK RECORD DESCRIPTION

Description of the track record structure as stored in the track file.
All word size values are stored as 'high byte first'

Byte    Type            Name        Description
------- --------------- ----------- -------------------------------------
0            unsigned char   status  0 No GPS (bit 5-7)
                                     1 No fix
                                     2 2d fix
                                     3 3d fix
                                     4 DGPS (3d) fix

0                            hour    hours of position fix (UTC) (bit0-4)

1            unsigned char   minute  minutes of position fix (UTC)
2            unsigned char   second  seconds of position fix (UTC)
3            unsigned char   decSecond 1/100 seconds of position fix (UTC)

4            unsigned char   sat     number of satellites being tracked
5            unsigned char   hdop    HDOP * 10 (0.1 precision)

6,7,8,9      long            lat     lat*10^7 (+ is north, - is south)
10,11,12,13  long            lon     lon*10^7 (+ is east, - is west)

14,15        unsigned short  speed   current speed in knots*10 (0.1 km/h prec.)
16,17        unsigned short  course  course made good*10 (0.1 deg precision)
18,19,20,21  long            alt     altitude in meter * 100 (0.01 meter precision)

************************************************************************/

#define trkStatus        0

#define trkHour          0
#define trkMinute        1
#define trkSecond        2
#define trkDecSecond     3

#define trkSat           4

#define trkHDOP          5

#define trkLatHiHi       6
#define trkLatHiLo       7
#define trkLatLoHi       8
#define trkLatLoLo       9

#define trkLonHiHi       10
#define trkLonHiLo       11
#define trkLonLoHi       12
#define trkLonLoLo       13

#define trkSpeedHi       14
#define trkSpeedLo       15

#define trkCourseHi      16
#define trkCourseLo      17

#define trkAltHiHi       18
#define trkAltHiLo       19
#define trkAltLoHi       20
#define trkAltLoLo       21

/************************************************************************
TRACK RECORD DATA VALID DESCRIPTION

If status is 'No GPS' or 'No fix' the time will be updated according to
the Palm internal clock. You should disregard all other values,

If one of the recorded track values is not available, the value is set
to a default 'data not available' value.

************************************************************************/

#define TrackPositionNotAvail  2000000000
#define TrackSpeedNotAvail     10000
#define TrackCourseNotAvail    4000
#define TrackTimeNotAvail      255    /* second & year vars only */
#define TrackDGPSNotAvail      255
#define TrackSatNotAvail       255
#define TrackHDOPNotAvail      255
#define TrackAltitudeNotAvail  -100000000

/************************************************************************
TRACK FILE INFORMATION RECORD DESCRIPTION

The first two records in the track log contains information about the
conditions under which the track was obtained.

Record 1

Byte    Type            Name        Description
------- --------------- ----------- -------------------------------------
0,1     char s[2]       ID          track ID ("CG")
2       char            version     track format version (3)
3       unsigned char   interval    User selected track save
                                      0 All positions
                                      1 Movements
                                      2 Every second
                                      3 Every 2 seconds
                                      4 Every 10 seconds
                                      5 Every minute
4,5     unsigned short  gps         GPS unit information
                                      0 No GPS detected (status is NoGps)
                                      1 Unknown GPS (generic)
                                      2 Demo mode enabled
                                      3 Generic Garmin GPS
                                      4 Generic Rockwell chipset GPS
                                      5 Generic Magellan GPS
                                      6 DeLorme Tripmate GPS
                                      7 Generic Motorola GPS
6       unsigned char   year        year of track start (UTC)
7       unsigned char   month       month of track start (UTC)
8       unsigned char   day         day of track start (UTC)
9       unsigned char   hour        hour of track start (UTC)
10      unsigned char   minute      minute of track start (UTC)
11      unsigned char   second      second of track start (UTC)
12      unsigned char   decSecond   1/100 seconds (UTC)
13      char            timeOffset  user preference UTC time offset (hours)
14-21   unused

Record 2

Byte    Type            Name        Description
------- --------------- ----------- -------------------------------------
0       char *          desc        first character in the description
                                    string 25 chars (continued in the
                                    next record)
Record 3

Byte    Type            Name        Description
------- --------------- ----------- -------------------------------------
0       char *          desc        description string (continued)
3-21    unused                                   
                               
************************************************************************/

#define trkInfoID1         0
#define trkInfoID2         1

#define trkInfoVersion     2

#define trkInfoInterval    3

#define trkInfoGPSHi       4
#define trkInfoGPSLo       5

#define trkInfoYear        6
#define trkInfoMonth       7
#define trkInfoDay         8

#define trkInfoHour        9
#define trkInfoMinute      10
#define trkInfoSecond      11
#define trkInfoDecSecond   12

#define trkInfoTimeOffset  13

/***********************************************************************/
